/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.qatest.ddl.auto.partition;

import org.junit.Ignore;
import org.junit.Test;
import org.junit.runners.Parameterized;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;

/**
 * @author chenghui.lch
 */

public class PartitionColumnTypeForRangeColumnsTest extends PartitionColumnTypeTestBase {

    public TestParameter parameter;

    public PartitionColumnTypeForRangeColumnsTest(TestParameter parameter) {
        super(parameter);
    }

    @Parameterized.Parameters(name = "{index}: partColTypeTestCase {0}")
    public static List<TestParameter> parameters() {
        // partition strategy: range/list/hash/range column/list column/key
        // data type: numeric/string/time
        return Arrays.asList(

            /**
             * ========= Range Columns =========== 
             */
            /**
             * Range Columns
             */
            /* datetime */
            new PartitionColumnTypeTestBase.TestParameter(
                "rc_datetime",
                new String[] {"c1"}/*col*/,
                new String[] {"datetime default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('2020-12-12 00:00:00')", "('2021-11-11 23:59:59')", "('2022-11-11 23:59:59')",
                    "('2023-11-11 23:59:59')"}/*bndVal*/,
                new String[] {
                    "(null)", "('2020-12-12 00:00:00')", "('2020-11-11 23:59:59.123')", "('2020-11-11 23:59:59.789')",
                    "('2020-12-11')", "('0000-00-00 00:00:00')", "('9999-99-99 99:99:99')", "('')"}/*insertValues*/,
                isMySQL80() ? new String[] {
                    "(null)", "('2020-12-12 00:00:00')", "('2020-11-11 23:59:59')", "('2020-11-12 00:00:00')",
                    "('2020-12-11')", "('0000-00-00 00:00:00')"} : new String[] {
                    "(null)", "('2020-12-12 00:00:00')", "('2020-11-11 23:59:59')", "('2020-11-12 00:00:00')",
                    "('2020-12-11')", "('0000-00-00 00:00:00')", "('9999-99-99 99:99:99')", "('')"}/*selectValues*/
                , isMySQL80() ? new String[] {
                "('0000-00-00 00:00:00')", "('2020-11-11 23:59:58.123')", "('2020-11-11 23:59:59.799')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00')"} : new String[] {
                "('')", "('0000-00-00 00:00:00')", "('2020-11-11 23:59:58.123')", "('2020-11-11 23:59:59.799')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00')", "('9999-99-99 99:99:99')"}/*rngSortValues*/
            )

            /* datetime(6) */
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_datetime6",
                new String[] {"c1"}/*col*/,
                new String[] {"datetime(6) default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('2020-12-12 00:00:00')", "('2021-11-11 23:59:59')", "('2022-11-11 23:59:59')",
                    "('2023-11-11 23:59:59')"}/*bndVal*/,
                new String[] {
                    "(null)", "('2020-12-12 00:00:00.123456')", "('2020-11-11 23:59:59.789013')",
                    "('2020-11-11 23:59:59.789')", "('2020-12-11')", "('0000-00-00 00:00:00')",
                    "('9999-99-99 99:99:99')", "('')"}/*insertValues*/
                ,
                new String[] {
                    "(null)", "('2020-12-12 00:00:00.123456')", "('2020-11-11 23:59:59.789013')",
                    "('2020-11-11 23:59:59.789')", "('2020-12-11')", "('0000-00-00 00:00:00')"}/*selectValues*/
                , isMySQL80() ? new String[] {
                "('0000-00-00 00:00:00')", "('2020-11-11 23:59:58.123')", "('2020-11-11 23:59:59.799')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00')"} : new String[] {
                "('')", "('0000-00-00 00:00:00')", "('2020-11-11 23:59:58.123')", "('2020-11-11 23:59:59.789013')",
                "('2020-11-11 23:59:59.799')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00.123456')", "('9999-99-99 99:99:99')"}/*rngSortValues*/
            )

            /* date */
            // '1000-01-01' to '9999-12-31'
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_date",
                new String[] {"c1"}/*col*/,
                new String[] {"date default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('2020-12-12')", "('2021-11-12')", "('2022-11-12')", "('2023-11-12')", "('9999-12-31')"}/*bndVal*/,
                new String[] {
                    "(null)", "('2020-12-12')", "('2020-11-11')", "('2020-12-11')", "('0000-00-00')", "('1000-00-00')",
                    "('1000-01-01')",
                    "('9999-12-30')", "('')"}/*insertValues*/,
                isMySQL80() ? new String[] {
                    "(null)", "('2020-12-12')", "('2020-11-11')", "('2020-12-11')", "('0000-00-00')", "('1000-00-00')",
                    "('1000-01-01')",
                    "('9999-12-31')",
                    "('0000-00-00 00:00:00')"} : new String[] {
                    "(null)", "('2020-12-12')", "('2020-11-11')", "('2020-12-11')", "('0000-00-00')", "('1000-00-00')",
                    "('1000-01-01')",
                    "('9999-12-31')",
                    "('9999-99-99')", "('0000-00-00 00:00:00')"}/*selectValues*/
                , isMySQL80() ? new String[] {
                "('0000-00-00 00:00:00')", "('2020-11-11 23:59:58.123')", "('2020-11-11 23:59:59.799')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00')"} : new String[] {
                "('')", "('0000-00-00 00:00:00')", "('1000-01-01')", "('2020-11-11 23:59:58')",
                "('2020-11-11 23:59:59')",
                "('2020-11-11 23:59:59')",
                "('2020-12-11 00:00:00')",
                "('2020-12-12 00:00:00')", "('9999-12-31')", "('9999-99-99 99:99:99')"}/*rngSortValues*/
            )

            /* BIGINT */
            //# bigint: -9223372036854775808 to 9223372036854775807
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_bigint",
                new String[] {"c1"}/*col*/,
                new String[] {"bigint default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(-9223372036854775803)",
                    "(-2147483653)",
                    "(-2147483648)",
                    "(-2147483643)",
                    "(-8388613)",
                    "(-8388608)",
                    "(-8388603)",
                    "(-32773)",
                    "(-32768)",
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(2147483652)",
                    "(4294967290)",
                    "(4294967295)",
                    "(4294967300)",
                    "(9223372036854775802)",
                    "(9223372036854775807)",
                    "(maxvalue)"}/*bndVal*/,
                new String[] {
                    "(0)", "(null)", "(-9223372036854775808)", "(9223372036854775807)", "(-9223372036854775807)",
                    "(922337203685477580)", "(922337203685477579)", "('-9223372036854775808')",
                    "('922337203685477580')", "('184467440737095516150')", "('-184467440737095516150')"}/*insertValues*/
                , new String[] {
                "(0)", "(null)", "(-9223372036854775808)", "(9223372036854775807)", "(-9223372036854775807)",
                "(922337203685477580)", "(922337203685477579)", "('-9223372036854775808')",
                "('922337203685477580')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)"
//                , "(9223372036854775812)", "(18446744073709551610)",
//                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
//                "('184467440737095516150')"
            }/*rnqQuerySortValues*/
            )

            /* BIGINT UNSINGED */
            //# bigint unsigned: 0~18446744073709551615
            // rnqQuerySortValues should add (18446744073709551620),(184467440737095516148),(184467440737095516150)
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_ubigint",
                new String[] {"c1"}/*col*/,
                new String[] {"bigint unsigned default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(2147483652)",
                    "(4294967290)",
                    "(4294967295)",
                    "(4294967300)",
                    "(9223372036854775802)",
                    "(9223372036854775807)",
                    "(maxvalue)"}/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(1)", "(18446744073709551615)", "(18446744073709551614)",
                    "('18446744073709551615')", "('184467440737095516150')",
                    "('-184467440737095516150')"}/*insertValues*/
                , new String[] {
                "(null)", "(0)", "(1)", "(18446744073709551615)", "(18446744073709551614)",
                "('18446744073709551615')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* INT */
            //# INT: -2147483648 to 2147483647
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_int",
                new String[] {"c1"}/*col*/,
                new String[] {"int default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(-2147483643)",
                    "(-8388613)",
                    "(-8388608)",
                    "(-8388603)",
                    "(-32773)",
                    "(-32768)",
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(maxvalue)"}/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(-2147483648)", "(2147483647)", "(-2147483647)", "(2147483646)",
                    "('-2147483648')", "('2147483647')", "('184467440737095516150')",
                    "('-184467440737095516150')"}/*insertValues*/
                , new String[] {
                "(null)", "(0)", "(-2147483648)", "(2147483647)", "(-2147483647)", "(2147483646)", "('-2147483648')",
                "('2147483647')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/

            )

            /* INT UNSIGNED */
            //# INT UNSIGNED: 0 to 4294967295
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_uint",
                new String[] {"c1"}/*col*/,
                new String[] {"int unsigned default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(2147483652)",
                    "(4294967290)",
                    "(4294967295)",
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(4294967295)", "(2147483647)", "(2147483646)", "('4294967295')", "('2147483647')",
                    "('184467440737095516150')", "('-184467440737095516150')"}/*insertValues*/
                , new String[] {
                "(null)", "(0)", "(4294967295)", "(2147483647)", "(2147483646)", "('4294967295')",
                "('2147483647')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* MEDIUMINT */
            //# MEDIUMINT: -8388608 to 8388607
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_mediumint",
                new String[] {"c1"}/*col*/,
                new String[] {"mediumint default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(-8388603)",
                    "(-32773)",
                    "(-32768)",
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(-8388608)", "(8388607)", "(-8388607)", "(8388606)", "('-8388608')",
                    "('8388607')", "('184467440737095516150')", "('-184467440737095516150')"}/*insertValues*/,
                new String[] {
                    "(null)", "(0)", "(-8388608)", "(8388607)", "(-8388607)", "(8388606)", "('-8388608')",
                    "('8388607')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* MEDIUMINT UNSIGNED*/
            //# MEDIUMINT UNSIGNED: 0 to 16777215
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_umediumint",
                new String[] {"c1"}/*col*/,
                new String[] {"mediumint unsigned default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(16777215)", "(8388606)",
                    "('8388607')", "('184467440737095516150')", "('-184467440737095516150')"}/*insertValues*/,
                new String[] {
                    "(null)", "(0)", "(16777215)", "(8388606)",
                    "('8388607')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* SMALLINT */
            //# SMALLINT: -32768 to 32767
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_smallint",
                new String[] {"c1"}/*col*/,
                new String[] {"smallint default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(maxvalue)"}/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(-32768)", "(32767)", "(-32767)", "(32766)", "('-32768')",
                    "('32767')", "('184467440737095516150')", "('-184467440737095516150')"}/*insertValues*/,
                new String[] {
                    "(null)", "(0)", "(-32768)", "(32767)", "(-32767)", "(32766)", "('-32768')",
                    "('32767')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* SMALLINT UNSIGNED */
            //# SMALLINT UNSIGNED: 0 to 65535
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_usmallint",
                new String[] {"c1"}/*col*/,
                new String[] {"smallint unsigned default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(65535)", "(1)", "(65534)", "('65535')", "('184467440737095516150')",
                    "('-184467440737095516150')",
                    "('-184467440737095516148')",
                    "('-9223372036854775813')",
                    "(-9223372036854775808)",
                    "(-9223372036854775803)",
                    "(-2147483653)",
                    "(-2147483648)",
                    "(-2147483643)",
                    "(-8388613)",
                    "(-8388608)",
                    "(-8388603)",
                    "(-32773)",
                    "(-32768)",
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(255)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(2147483652)",
                    "(4294967290)",
                    "(4294967295)",
                    "(4294967300)",
                    "(9223372036854775802)",
                    "(9223372036854775807)",
                    "(9223372036854775812)",
                    "(18446744073709551610)",
                    "(18446744073709551615)",
                    "('1844674407370955162')",
                    "('184467440737095516148')",
                    "('184467440737095516150')"}/*insertValues*/,
                new String[] {
                    "(null)", "(0)", "(65535)", "(1)", "(65534)", "('65535')"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* TINYINT */
            //# TINYINT: -128 to 127
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_tinyint",
                new String[] {"c1"}/*col*/,
                new String[] {"tinyint default null"}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)", "(0)", "(-128)", "(127)", "('184467440737095516150')", "('-184467440737095516150')",
                    "('-184467440737095516150')",
                    "('-184467440737095516148')",
                    "('-9223372036854775813')",
                    "(-9223372036854775808)",
                    "(-9223372036854775803)",
                    "(-2147483653)",
                    "(-2147483648)",
                    "(-2147483643)",
                    "(-8388613)",
                    "(-8388608)",
                    "(-8388603)",
                    "(-32773)",
                    "(-32768)",
                    "(-32763)",
                    "(-133)",
                    "(-128)",
                    "(-123)",
                    "(-5)",
                    "(0)",
                    "(5)",
                    "(122)",
                    "(127)",
                    "(132)",
                    "(255)",
                    "(32762)",
                    "(32767)",
                    "(32772)",
                    "(65530)",
                    "(65535)",
                    "(65540)",
                    "(8388602)",
                    "(8388607)",
                    "(8388612)",
                    "(16777210)",
                    "(16777215)",
                    "(16777220)",
                    "(2147483642)",
                    "(2147483647)",
                    "(2147483652)",
                    "(4294967290)",
                    "(4294967295)",
                    "(4294967300)",
                    "(9223372036854775802)",
                    "(9223372036854775807)",
                    "(9223372036854775812)",
                    "(18446744073709551610)",
                    "(18446744073709551615)",
                    "('1844674407370955162')",
                    "('184467440737095516148')",
                    "('184467440737095516150')"}/*insertValues*/,
                new String[] {
                    "(null)", "(0)", "(-128)", "(127)"}/*selectValues*/
                , new String[] {
                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
                "('184467440737095516150')"}/*rnqQuerySortValues*/
            )

            /* TINYINT */
            //# TINYINT: 0 to 255
//            , new PartitionColumnTypeTestBase.TestParameter(
//                "rc_utinyint",
//                new String[] {"c1"}/*col*/,
//                new String[] {"tinyint unsigned default null"}/*data types*/,
//                new String[] {"set sql_mode='';set names utf8;", "", ""}/*prepStmts*/,
//                "range columns"/*strategy*/,
//                new String[] {
//                    "(5)",
//                    "(122)",
//                    "(127)",
//                    "(132)",
//                    "(255)",
//                    "(maxvalue)"
//                }/*bndVal*/,
//                new String[] {
//                    "(null)", "(0)", "(255)", "('255')", "('184467440737095516150')", "('-184467440737095516150')",
//                    "('-184467440737095516150')",
//                    "('-184467440737095516148')",
//                    "('-9223372036854775813')",
//                    "(-9223372036854775808)",
//                    "(-9223372036854775803)",
//                    "(-2147483653)",
//                    "(-2147483648)",
//                    "(-2147483643)",
//                    "(-8388613)",
//                    "(-8388608)",
//                    "(-8388603)",
//                    "(-32773)",
//                    "(-32768)",
//                    "(-32763)",
//                    "(-133)",
//                    "(-128)",
//                    "(-123)",
//                    "(-5)",
//                    "(0)",
//                    "(5)",
//                    "(122)",
//                    "(127)",
//                    "(132)",
//                    "(255)",
//                    "(32762)",
//                    "(32767)",
//                    "(32772)",
//                    "(65530)",
//                    "(65535)",
//                    "(65540)",
//                    "(8388602)",
//                    "(8388607)",
//                    "(8388612)",
//                    "(16777210)",
//                    "(16777215)",
//                    "(16777220)",
//                    "(2147483642)",
//                    "(2147483647)",
//                    "(2147483652)",
//                    "(4294967290)",
//                    "(4294967295)",
//                    "(4294967300)",
//                    "(9223372036854775802)",
//                    "(9223372036854775807)",
//                    "(9223372036854775812)",
//                    "(18446744073709551610)",
//                    "(18446744073709551615)",
//                    "('1844674407370955162')",
//                    "('184467440737095516148')",
//                    "('184467440737095516150')"}/*insertValues*/,
//                new String[] {
//                    "(null)", "(0)", "(255)", "('255')",
//                    "('-184467440737095516150')",
//                    "('-184467440737095516148')",
//                    "('-9223372036854775813')",
//                    "(-9223372036854775808)",
//                    "(-9223372036854775803)",
//                    "(-2147483653)",
//                    "(-2147483648)",
//                    "(-2147483643)",
//                    "(-8388613)",
//                    "(-8388608)",
//                    "(-8388603)",
//                    "(-32773)",
//                    "(-32768)",
//                    "(-32763)",
//                    "(-133)",
//                    "(-128)",
//                    "(-123)",
//                    "(-5)",
//                    "(0)",
//                    "(5)",
//                    "(122)",
//                    "(127)",
//                    "(132)",
//                    "(255)",
//                    "(32762)",
//                    "(32767)",
//                    "(32772)",
//                    "(65530)",
//                    "(65535)",
//                    "(65540)",
//                    "(8388602)",
//                    "(8388607)",
//                    "(8388612)",
//                    "(16777210)",
//                    "(16777215)",
//                    "(16777220)",
//                    "(2147483642)",
//                    "(2147483647)",
//                    "(2147483652)",
//                    "(4294967290)",
//                    "(4294967295)",
//                    "(4294967300)",
//                    "(9223372036854775802)",
//                    "(9223372036854775807)",
//                    "(9223372036854775812)",
//                    "(18446744073709551610)",
//                    "(18446744073709551615)",
//                    "('1844674407370955162')",
//                    "('184467440737095516148')",
//                    "('184467440737095516150')"}/*selectValues*/
//                , new String[] {
//                "(-184467440737095516150)", "(-184467440737095516148)", "(-9223372036854775813)",
//                "(-9223372036854775808)", "(-9223372036854775803)", "(-2147483653)", "(-2147483648)", "(-2147483643)",
//                "(-8388613)", "(-8388608)", "(-8388603)", "(-32773)", "(-32768)", "(-32763)", "(-133)", "(-128)",
//                "(-123)", "(-5)", "(0)", "(5)", "(122)", "(127)", "(132)", "(32762)", "(32767)", "(32772)", "(65530)",
//                "(65535)", "(65540)", "(8388602)", "(8388607)", "(8388612)", "(16777210)", "(16777215)", "(16777220)",
//                "(2147483642)", "(2147483647)", "(2147483652)", "(4294967290)", "(4294967295)", "(4294967300)",
//                "(9223372036854775802)", "(9223372036854775807)", "(9223372036854775812)", "(18446744073709551610)",
//                "(18446744073709551615)", "('18446744073709551620')", "('184467440737095516148')",
//                "('184467440737095516150')"}/*rnqQuerySortValues*/
//            )

            /* varchar */
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_varchar8",
                new String[] {"c1"}/*col*/,
                new String[] {"varchar(8) CHARACTER SET utf8 COLLATE utf8_general_ci default null "}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "", ""},
                "range columns"/*strategy*/,
                new String[] {
                    "('')",
                    "('12345678')",
                    "('c')",
                    "('cCc')",
                    "('z   ')",
                    "('zZz   ')",
                    "('☺')",
                    "('世')",
                    "(x'E4B896E7958C')",
                    genStringHexBinaryByCharset("世界人", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "gbk"),
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('☺')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    "('世界')",
                    "( x'E4B896E7958C' )",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")
                }/*insertValues*/
                ,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('ccc')",
                    "('CcC')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('☺')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    "('世界')",
                    "( x'E4B896E7958C' )",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "gbk")}/*selectValues*/
                , new String[] {
                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')", "('世界')",
                "('世界人民')"}/*rngSortValues*/
            )

            /* fix by ??? */
            /* varchar */
//            , new PartitionColumnTypeTestBase.TestParameter(
//                "rc_varchar8_gbk",
//                new String[] {"c1"}/*col*/,
//                new String[] {"varchar(8) CHARACTER SET gbk default null "}/*data types*/,
//                new String[] {"set sql_mode='';set names gbk;", "set names gbk;", "set names utf8;"}/*prepStmts*/,
//                "range columns"/*strategy*/,
//                new String[] {
//                    "('')",
//                    "('12345678')",
//                    "('c')",
//                    "('cCc')",
//                    "('z   ')",
//                    "('zZz   ')",
//                    genStringHexBinaryByCharset("世", "gbk"),
//                    genStringHexBinaryByCharset("世界", "gbk"),
//                    genStringHexBinaryByCharset("世界人", "gbk"),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk"),
//                    "(maxvalue)"
//                }/*bndVal*/,
//                new String[] {
//                    "(null)",
//                    "('')",
//                    "('a')",
//                    "('b')",
//                    "('A')",
//                    "('B')",
//                    "('Z     ')",
//                    "('z  ')",
//                    "('z')",
//                    "('ccc')",
//                    "('CcC')",
//                    "('12345678')",
//                    "('1234567889ABC')",
//                    "('1234567889EFG')",
//                    genStringHexBinaryByCharset("世", "gbk"),
//                    genStringHexBinaryByCharset("世界", "gbk"),
//                    genStringHexBinaryByCharset("世界人民", "gbk"),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk")
//                }/*insertValues*/
//                ,
//                new String[] {
//                    "(null)",
//                    "('')",
//                    "('a')",
//                    "('b')",
//                    "('A')",
//                    "('B')",
//                    "('ccc')",
//                    "('CcC')",
//                    "('Z     ')",
//                    "('z  ')",
//                    "('z')",
//                    "('12345678')",
//                    "('1234567889ABC')",
//                    "('1234567889EFG')",
//                    genStringHexBinaryByCharset("世", "gbk"),
//                    genStringHexBinaryByCharset("世界", "gbk"),
//                    genStringHexBinaryByCharset("世界人民", "gbk"),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk")
//                }/*selectValues*/
//                , new String[] {
//                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')",
//                genStringHexBinaryByCharset("世界", "gbk"),
//                genStringHexBinaryByCharset("世界人民", "gbk")}/*rngSortValues*/
//            )

            /* char */
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_char8",
                new String[] {"c1"}/*col*/,
                new String[] {"char(8) CHARACTER SET utf8 COLLATE utf8_general_ci default null "}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "set names utf8;", "set names utf8;"}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('')",
                    "('12345678')",
                    "('c')",
                    "('cCc')",
                    "('z   ')",
                    "('zZz   ')",
                    "('☺')",
                    "('世')",
                    "(x'E4B896E7958C')",
                    genStringHexBinaryByCharset("世界人", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8"),
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")}/*insertValues*/
                ,
                new String[] {
                    "('A')", "('B')", "('a')", "('b')", "('a  ')", "('b   ')", "(\'\')", "('世界')",
                    "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)",
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('☺')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    "('世界')",
                    "( x'E4B896E7958C' )",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")}/*selectValues*/
                , new String[] {
                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')", "('世界')",
                "('世界人民')"}/*rngSortValues*/
            )

            /* char */
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_char16",
                new String[] {"c1"}/*col*/,
                new String[] {"char(16) CHARACTER SET utf8 COLLATE utf8_bin default null "}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "set names utf8;", "set names utf8;"}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('')",
                    "('12345678')",
                    "('c')",
                    "('cCc')",
                    "('z   ')",
                    "('zZz   ')",
                    "('☺')",
                    "('世')",
                    "(x'E4B896E7958C')",
                    genStringHexBinaryByCharset("世界人", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8"),
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")}/*insertValues*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('☺')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    "('世界')",
                    "( x'E4B896E7958C' )",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")}/*selectValues*/
                , new String[] {
                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')", "('世界')",
                "('世界人民')"}/*rngSortValues*/
            )

            /* rc_char16_utf8 */
            , new PartitionColumnTypeTestBase.TestParameter(
                "rc_char16_utf8",
                new String[] {"c1"}/*col*/,
                new String[] {"char(16) CHARACTER SET utf8 default null "}/*data types*/,
                new String[] {"set sql_mode='';set names utf8;", "set names utf8;", "set names utf8;"}/*prepStmts*/,
                "range columns"/*strategy*/,
                new String[] {
                    "('')",
                    "('12345678')",
                    "('c')",
                    "('cCc')",
                    "('z   ')",
                    "('zZz   ')",
                    "('☺')",
                    "('世')",
                    "(x'E4B896E7958C')",
                    genStringHexBinaryByCharset("世界人", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8"),
                    "(maxvalue)"
                }/*bndVal*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    genStringHexBinaryByCharset("世界人民", "utf8", false, false),
                    genStringHexBinaryByCharset("世界", "utf8", false, false),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8", false, false)}/*insertValues*/,
                new String[] {
                    "(null)",
                    "('')",
                    "('a')",
                    "('b')",
                    "('A')",
                    "('B')",
                    "('☺')",
                    "('Z     ')",
                    "('z  ')",
                    "('z')",
                    "('ccc')",
                    "('CcC')",
                    "('12345678')",
                    "('1234567889ABC')",
                    "('1234567889EFG')",
                    "('世界')",
                    "( x'E4B896E7958C' )",
                    genStringHexBinaryByCharset("世界人民", "utf8"),
                    genStringHexBinaryByCharset("世界", "utf8"),
                    genStringHexBinaryByCharset("世界人民万岁", "utf8")}/*selectValues*/
                , new String[] {
                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')", "('世界')",
                "('世界人民')"}/*rngSortValues*/
            )

            /**
             * PolarDB-X now is not supported the predicate process with [_charset_name] literal [COLLATE collation_name]
             * because of the dynamic params will miss the charset info of _charset_name, so ignore the case
             */
//            /* rc_char16_gbk */
//            , new PartitionColumnTypeTestBase.TestParameter(
//                "rc_char16_gbk",
//                new String[] {"c1"}/*col*/,
//                new String[] {"char(16) CHARACTER SET gbk default null "}/*data types*/,
//                new String[] {"set sql_mode='';set names gbk;", "set names gbk;", "set names gbk;"}/*prepStmts*/,
//                "range columns"/*strategy*/,
//                new String[] {
//                    "('')",
//                    "('12345678')",
//                    "('c')",
//                    "('cCc')",
//                    "('z   ')",
//                    "('zZz   ')",
//                    genStringHexBinaryByCharset("世界", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false),
//                    "(maxvalue)"
//                }/*bndVal*/,
//                new String[] {
//                    "(null)",
//                    "('')",
//                    "('a')",
//                    "('b')",
//                    "('A')",
//                    "('B')",
//                    "('Z     ')",
//                    "('z  ')",
//                    "('z')",
//                    "('ccc')",
//                    "('CcC')",
//                    "('12345678')",
//                    "('1234567889ABC')",
//                    "('1234567889EFG')",
//                    genStringHexBinaryByCharset("世界", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false)}/*insertValues*/,
//                new String[] {
//                    "(null)",
//                    "('')",
//                    "('a')",
//                    "('b')",
//                    "('A')",
//                    "('B')",
//                    "('Z     ')",
//                    "('z  ')",
//                    "('z')",
//                    "('ccc')",
//                    "('CcC')",
//                    "('12345678')",
//                    "('1234567889ABC')",
//                    "('1234567889EFG')",
//                    "('世界')",
//                    "( x'E4B896E7958C' )",
//                    genStringHexBinaryByCharset("世界", "gbk"),
//                    genStringHexBinaryByCharset("世界人民", "gbk"),
//                    genStringHexBinaryByCharset("世界人民万岁", "gbk")}/*selectValues*/
//                , new String[] {
//                "('12345677')", "('12345679')", "('A')", "('B')", "('a')", "('a  ')", "('b')", "('b   ')", "('世界')",
//                "('世界人民')"}/*rngSortValues*/
//            )

            /* fix by ??? */
            /* rc_char16_gbk_utf8 */
//            , new PartitionColumnTypeTestBase.TestParameter(
//                "rc_char16_gbk_utf8",
//                new String[] {"c1", "c2"}/*col*/,
//                new String[] {
//                    "char(16) CHARACTER SET gbk default null",
//                    "char(16) CHARACTER SET utf8 default null"}/*data types*/,
//                new String[] {"set sql_mode='';set names gbk;", "set names gbk;", "set names gbk;"}/*prepStmts*/,
//                "range columns"/*strategy*/,
//                new String[] {
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民万岁", "utf8", false, false)),
//                    "(maxvalue,maxvalue)"
//                }/*bndVal*/,
//                new String[] {
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民万岁", "utf8", false, false))}/*insertValues*/,
//                new String[] {
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民", "utf8", false, false)),
//                    String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false),
//                        genStringHexBinaryByCharset("世界人民万岁", "utf8", false, false))}/*selectValues*/
//                , new String[] {
//                String.format("(%s,%s)", genStringHexBinaryByCharset("世界", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界", "utf8", false, false)),
//                String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民", "utf8", false, false)),
//                String.format("(%s,%s)", genStringHexBinaryByCharset("世界人民万岁", "gbk", false, false),
//                    genStringHexBinaryByCharset("世界人民万岁", "utf8", false, false))}/*rngSortValues*/
//            )

//            /* varchar */
//            ,new PartitionColumnTypeTestBase.TestParameter(
//            "rngcol_varchar",
//            new String[] {"c1"}/*col*/,
//            new String[] {"varchar(8) CHARACTER SET utf8 COLLATE utf8_general_ci default null "}/*data types*/,
//            "key"/*strategy*/,
//            new String[] {"16"}/*bndVal*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*insertValues*/
//            ,
//            new String[] {
//                "('A')", "('B')", "('a')", "('b')", "('a  ')", "('b   ')", "(\'\')", "('世界')",
//                "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*se lectValues*/
//        )
//
//            /* varchar */
//            , new PartitionColumnTypeTestBase.TestParameter(
//            "key_varchar32",
//            new String[] {"c1"}/*col*/,
//            new String[] {"varchar(32) CHARACTER SET utf8 COLLATE utf8_bin default null "}/*data types*/,
//            "key"/*strategy*/,
//            new String[] {"16"}/*bndVal*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*insertValues*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*selectValues*/
//        )
//
//            /* char */
//            , new PartitionColumnTypeTestBase.TestParameter(
//            "rngcol_char32",
//            new String[] {"c1"}/*col*/,
//            new String[] {"char(32) CHARACTER SET utf8 COLLATE utf8_general_ci default null "}/*data types*/,
//            "key"/*strategy*/,
//            new String[] {"16"}/*bndVal*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*insertValues*/
//            ,
//            new String[] {
//                "('A')", "('B')", "('a')", "('b')", "('a  ')", "('b   ')", "(\'\')", "('世界')",
//                "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*selectValues*/
//        )
//
//            /* char */
//            , new PartitionColumnTypeTestBase.TestParameter(
//            "rngcol_char32",
//            new String[] {"c1"}/*col*/,
//            new String[] {"char(32) CHARACTER SET utf8 COLLATE utf8_bin default null "}/*data types*/,
//            "key"/*strategy*/,
//            new String[] {"16"}/*bndVal*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*insertValues*/,
//            new String[] {
//                "('a')", "('b')", "(\'\')", "('世界')", "( x'E4B896E7958C' )", "('☺')", "('12345678')", "(null)"}/*selectValues*/
//        )

        );
    }

    @Test
    public void runTest() throws SQLException {
        super.testInsertAndSelect();
    }

}
